<?php
namespace Zeedhi\PaymentPlatform\Implementations\Braspag;

use Zeedhi\PaymentPlatform\DebitBuy\DebitBuyPlatform;
use Zeedhi\PaymentPlatform\DebitBuy\DebitBuyTransaction;

use Zeedhi\PaymentPlatform\BilletBuy\BilletBuyPlatform;
use Zeedhi\PaymentPlatform\BilletBuy\BilletBuyTransaction;

use Zeedhi\PaymentPlatform\CreditCard;
use Zeedhi\PaymentPlatform\Customer;
use Zeedhi\PaymentPlatform\Address;
use Zeedhi\PaymentPlatform\Item;
use Zeedhi\PaymentPlatform\RequestURL;
use Zeedhi\PaymentPlatform\Exception;

class PaymentImpl implements DebitBuyPlatform, BilletBuyPlatform{

    const SANDBOX = "SANDBOX";
    const ENDPOINT_SANDBOX = "https://apisandbox.braspag.com.br/";

    const PRODUCTION = "PRODUCTION";
    const ENDPOINT_PRODUCTION = "https://api.braspag.com.br/";

    const POST = "POST";

    /** @var String */
    private $merchantKey;

    /** @var array */
    private $requestHeaders;

    /** @var HttpRequest */
    private $httpRequest;

    /** @var string */
    private $urlRequest;

    public function __construct($merchantKey, $merchantId, HttpRequest $httpRequest, $urlRequest) {

        $this->merchantKey = $merchantKey;
        $this->merchantId = $merchantId;
        $this->httpRequest = $httpRequest;
        $this->urlRequest = $urlRequest;
        $this->requestHeaders = array(
            'Content-Type: application/json',
            'MerchantId: '.$this->merchantId,
            'Merchantkey: '.$this->merchantKey
        );

    }

    public function formatDebitCustomer($customer){
        $customerArray['Name'] = $customer->getName();
        $address = $customer->getAddress();
        if($address !== null){
            $customerArray['Address'] = array(
                'Street' => $address->getStreet(),
                'Number' => $address->getNumber(),
                'Complement' => $address->getComplement(),
                'ZipCode' => $address->getZipCode(),
                'City' => $address->getCity(),
                'State' => $address->getState(),
                'Country' => $address->getCountry(),
                'District' => $address->getNeighborhood()
            );
            $customerArray['DeliveryAddress'] = array(
                'Street' => $address->getStreet(),
                'Number' => $address->getNumber(),
                'Complement' => $address->getComplement(),
                'ZipCode' => $address->getZipCode(),
                'City' => $address->getCity(),
                'State' => $address->getState(),
                'Country' => $address->getCountry(),
                'District' => $address->getNeighborhood()
            );
        }

        return $customerArray;
    }

    public function formatBilletCustomer($customer){
        $customerArray['Name'] = $customer->getName();
        $customerArray['Identity'] = $customer->getDocument();
        $customerArray['IdentityType'] = 'CPF';
        $address = $customer->getAddress();
        if($address !== null){
            $customerArray['Address'] = array(
                'Street' => $address->getStreet(),
                'Number' => $address->getNumber(),
                'Complement' => $address->getComplement(),
                'ZipCode' => $address->getZipCode(),
                'City' => $address->getCity(),
                'State' => $address->getState(),
                'Country' => $address->getCountry(),
                'District' => $address->getNeighborhood()
            );
        }

        return $customerArray;
    }

    public function formatDebitPayment($transaction){
        $paymentArray['Provider'] = $transaction->getProvider();
        $paymentArray['Type'] = $transaction->getTransactionType();
        $paymentArray['Amount'] = $transaction->getValue();
        $paymentArray['Currency'] = 'BRL';
        $paymentArray['Installments'] = 1;
        $paymentArray['Capture'] = false;
        $paymentArray['Authenticate'] = $transaction->getAuthentication();
        if($paymentArray['Authenticate']){
            $paymentArray['ReturnUrl'] = $transaction->getReturnUrl();
        }
        $paymentArray['DebitCard'] = array(
            'CardNumber' => $transaction->getCreditCard()->getNumbers(),
            'Holder' => $transaction->getCreditCard()->getHolderName(),
            'ExpirationDate' => $transaction->getCreditCard()->getExpDateMonth().'/'.$transaction->getCreditCard()->getExpDateYear(),
            'SecurityCode' => $transaction->getCreditCard()->getCvv(),
            'Brand' => $transaction->getCreditCard()->getFlag(),
            'SaveCard' => 'false'
        );
        $paymentArray['Credentials'] = array(
            'code' => $transaction->getCredentialsCode(),
            'key' => $transaction->getCredentialsKey()
        );
        
        return $paymentArray;
    }

    public function formatBilletPayment($transaction){
        $paymentArray['Provider'] = $transaction->getProvider();
        $paymentArray['Type'] = $transaction->getTransactionType();
        $paymentArray['Amount'] = $transaction->getValue();
        $paymentArray['Credentials'] = array(
            'Agency' => $transaction->getCredentialsAgency(),
            'Account' => $transaction->getCredentialsAccount(),
            'Code' => $transaction->getCredentialsCode(),
            'Agreement' => $transaction->getCredentialsAgreement()
        );
        
        return $paymentArray;
    }

    public function formatFraudAnalysis($transaction){
        $fraudAnalysisArray['Sequence'] = 'AnalyseFirst';
        $fraudAnalysisArray['Provider'] = 'Cybersource';
        $fraudAnalysisArray['SequenceCriteria'] = 'Always';
        $fraudAnalysisArray['TotalOrderAmount'] = $transaction->getValue();
        $fraudAnalysisArray['FingerPrintId'] = ''; /* Verificar como pegar */
        $fraudAnalysisArray['Browser'] = array(
            'CookiesAccepted' => false,
            'IpAddress' => '' /* Verificar como pegar */
        );
        $fraudAnalysisArray['Cart']['Items'] = $transaction->getItems();
        
        return $fraudAnalysisArray;
    }

    public function DebitBuyTransaction(DebitBuyTransaction $DebitBuyTransaction) {
        $request = array(
            "MerchantOrderId" => $DebitBuyTransaction->getTransactionId(),
            "Customer" => $this->formatDebitCustomer($DebitBuyTransaction->getCustomer()),
            "Payment" => $this->formatDebitPayment($DebitBuyTransaction)
        );
        $responseData = $this->createRequest(RequestURL::BRASPAG_SALES, self::POST, $request);
        if($responseData['response']['Payment']['Status'] === 2){
            return array(
                'status' => $responseData['Payment']['Status'],
                'message' => $responseData['Payment']['ProviderReturnMessage'],
                'urls' => $responseData['Payment']['Links'],
                'AuthenticationUrl' => $responseData['Payment']['AuthenticationUrl'],
                'paymentId' => $responseData['Payment']['PaymentId'],
                'nsu' => $responseData['Payment']['ProofOfSale']
            );
        } else{
            $this->treatDebitError($responseData);
        }
    }

    public function VoucherBuyTransaction(DebitBuyTransaction $DebitBuyTransaction) {
        $request = array(
            "MerchantOrderId" => $DebitBuyTransaction->getTransactionId(),
            "Customer" => $this->formatDebitCustomer($DebitBuyTransaction->getCustomer()),
            "Payment" => $this->formatDebitPayment($DebitBuyTransaction)
        );
        $responseData = $this->createRequest(RequestURL::BRASPAG_SALES, self::POST, $request);
        $response = array(
            'status' => $responseData['response']['Payment']['Status'],
            'message' => $responseData['response']['Payment']['ProviderReturnMessage'],
            'returnCode' => $responseData['response']['Payment']['ProviderReturnCode'],
            'urls' => $responseData['response']['Payment']['Links'],
            'paymentId' => $responseData['response']['Payment']['PaymentId']
        );
        if($responseData['response']['Payment']['Status'] === 2){
            $response['nsu'] = $responseData['response']['Payment']['ProofOfSale'];
        }

        return $response;
    }

    public function BilletBuyTransaction(BilletBuyTransaction $BilletBuyTransaction) {
        $request = array(
            "MerchantOrderId" => $BilletBuyTransaction->getTransactionId(),
            "Customer" => $this->formatBilletCustomer($BilletBuyTransaction->getCustomer()),
            "Payment" => $this->formatBilletPayment($BilletBuyTransaction)
        );
        $responseData = $this->createRequest(RequestURL::BRASPAG_SALES, self::POST, $request);
        return array(
            'billetNumber' => $responseData['response']['Payment']['BoletoNumber'],
            'billetBarCode' => $responseData['response']['Payment']['BarCodeNumber'],
            'urls' => $responseData['response']['Payment']['Links'],
            'paymentId' => $responseData['response']['Payment']['PaymentId'],
            'billetUrl' => $responseData['response']['Payment']['Url'],
            'status' => $responseData['response']['Payment']['Status']
        );
    }

    public function captureTransaction($url, $paymentId){
        
    }

    private function createRequest($url, $verb, $fields){
        $this->configCurl($url);
        if($this->urlRequest == self::ENDPOINT_SANDBOX) {
            $this->httpRequest->setOption(CURLOPT_PROXY, "http://192.168.122.3:8080");
            $this->httpRequest->setOption(CURLOPT_PROXYUSERPWD, "heitorcarvalho:teknisa3");
            $this->httpRequest->setOption(CURLOPT_SSL_VERIFYPEER, false);
        }
        $this->httpRequest->setOption(CURLOPT_CUSTOMREQUEST, $verb);
        $this->httpRequest->setOption(CURLOPT_POSTFIELDS, json_encode($fields));
        return $this->responseTreatment();
    }

    private function configCurl($url) {
        $this->httpRequest->setUrl($this->urlRequest.$url);
        $this->httpRequest->setOption(CURLOPT_HTTPHEADER, $this->requestHeaders);
        $this->httpRequest->setOption(CURLOPT_RETURNTRANSFER, true);
    }

    private function responseTreatment() {
        $response = json_decode($this->httpRequest->execute(), true);
        $httpStatusCode = $this->httpRequest->getInfo(CURLINFO_HTTP_CODE);
        $return = array();
        $return['response'] = $response;
        $return['httpStatusCode'] = $httpStatusCode;
        $this->httpRequest->close();
        if($httpStatusCode == '201' || $httpStatusCode == '200') {
            return $return;
        } else {
            throw Exception::apiBraspag($httpStatusCode, $response[0]['Message']); /* Verificar extrutura da mensagem de erro */
        }
    }

    public static function factory($merchantKey, $merchantId, $flag) {
        if($flag === self::SANDBOX){
            $urlRequest = self::ENDPOINT_SANDBOX;
        }else if($flag === self::PRODUCTION){
            $urlRequest = self::ENDPOINT_PRODUCTION;
        }
        return new self($merchantKey, $merchantId, new HttpRequest, $urlRequest);
    }
}