describe("TemplateManager", function() {

	var zhPromise = new ZHPromise(Q);
	var templateManager;

	beforeEach(function() {
		var window = {};
		window.templateUrl = "../../frontend/templates/dyman/";
		window.metadataUrl = "./";
		window.serviceUrl = "../backend/service/index.php";
		window.serviceUrls = [{
			"package": "identity",
			"baseUrl": "http://identity.zeedhi.com/backend/service/index.php",
		}, {
			"package": "pm",
			"baseUrl": "http://pm.zeedhi.com/backend/service/index.php"
		}, {
			"package": "devOps",
			"baseUrl": "http://devops.zeedhi.com/backend/service/index.php"
		}];

		window.metadataUrls = [{
			"package": "identity",
			"baseUrl": "http://identity.zeedhi.com/frontend/",
			"hasAcl": true,
			"hasWords": true
		}, {
			"package": "pm",
			"baseUrl": "http://pm.zeedhi.com/frontend/",
			"hasAcl": true
		}, {
			"package": "devOps",
			"baseUrl": "http://devops.zeedhi.com/frontend/",
			"hasWords": true,
			"hasSharedData": true
		}];
		templateManager = new TemplateManager(window, zhPromise);
	});

	it("Method getTemplateDefaultFolder should return the templateUrl folder", function() {
		var templateFolder = templateManager.getTemplateDefaultFolder();
		expect(templateFolder).toEqual("../../frontend/templates/dyman/");
	});

	it("Method getTemplateDefaultFolder should return the templateUrl concatenated with dyman if the templateUrl doesn't have a templates suffix", function() {
		var window = {};
		window.templateUrl = "../../frontend/templates/";
		window.metadataUrl = "./";
		window.serviceUrl = "../backend/service/index.php";
		var templateManager = new TemplateManager(window, zhPromise);

		var templateFolder = templateManager.getTemplateDefaultFolder();
		expect(templateFolder).toEqual(window.templateUrl + "dyman/");
	});

	it("Method getTemplateDefaultFolder should return the templateUrl if there is no window.templateUrl and there are window.templateUrls", function() {
		var window = {};
		window.templateUrls = [{
			package: "default",
			baseUrl: "../../frontend/templates/dyman/"
		}];
		window.metadataUrl = "./";
		window.serviceUrl = "../backend/service/index.php";
		var templateManager = new TemplateManager(window, zhPromise);

		var templateFolder = templateManager.getTemplateDefaultFolder();
		expect(templateFolder).toEqual("../../frontend/templates/dyman/");
	});

	it("The TemplateManager constructor should throw an exception if there is no window.templateUrl and there is no default templateUrl inside window.templateUrls", function() {
		var window = {};
		window.templateUrls = [{
			package: "jose",
			baseUrl: "../../frontend/templates/dyman/"
		}];
		window.metadataUrl = "./";
		window.serviceUrl = "../backend/service/index.php";

		expect(function() {
			return new TemplateManager(window, zhPromise);
		}).toThrow(new Error("The templateUrls must have a default template when there is not templateUrl"));
	});

	it("Method getTemplateURL should throw an exception if there is not a baseUrl defined for the package", function() {
		var window = {};
		window.templateUrls = [{
			package: "jose",
			baseUrl: "../../frontend/templates/dyman/"
		}, {
			package: "default",
			baseUrl: "../../frontend/templates/dyman/"
		}];
		window.metadataUrl = "./";
		window.serviceUrl = "../backend/service/index.php";
		var templateManager = new TemplateManager(window, zhPromise);

		expect(function() {
			return templateManager.getTemplateURL('jonDoe#widget/kanban.html');
		}).toThrow(new Error("There is no templateUrl with package jonDoe"));
	});

	it("Method getTemplateURL should return the url of the template if it's a custom templateUrl", function() {
		var window = {};
		window.templateUrl = "../../frontend/templates/dyman/";
		window.templateUrls = [{
			package: "something",
			baseUrl: "./something/"
		}];
		window.metadataUrl = "./";
		window.serviceUrl = "../backend/service/index.php";
		var templateManager = new TemplateManager(window, zhPromise);

		var template = templateManager.getTemplateURL("something#widget/kanban.html");
		expect(template).toEqual("./something/widget/kanban.html");
	});

	it("Method getTemplateURL should use the default package if a templateUrl was provided without #", function() {
		var window = {};
		window.templateUrl = "../../frontend/templates/dyman/";
		window.metadataUrl = "./";
		window.serviceUrl = "../backend/service/index.php";
		var templateManager = new TemplateManager(window, zhPromise);

		var template = templateManager.getTemplateURL("something.html");
		expect(template).toEqual("../../frontend/templates/dyman/something.html");
	});

	it("Method getTemplateURL should should return undefined if the parameter is empty", function() {
		var window = {};
		window.templateUrl = "../../frontend/templates/dyman/";
		window.metadataUrl = "./";
		window.serviceUrl = "../backend/service/index.php";
		var templateManager = new TemplateManager(window, zhPromise);

		var template = templateManager.getTemplateURL(null);
		expect(template).toEqual(undefined);
	});

	it("Method getFrameworkTemplateURL should return the url of the template if it's a framework templateUrl", function() {
		var window = {};
		window.templateUrl = "../../frontend/templates/dyman/";
		window.templateUrls = [{
			package: "something",
			baseUrl: "./something/"
		}];
		window.metadataUrl = "./";
		window.serviceUrl = "../backend/service/index.php";
		var templateManager = new TemplateManager(window, zhPromise);
		var template = templateManager.getFrameworkTemplateURL("widget/kanban.html");
		expect(template).toEqual("../../frontend/templates/dyman/widget/kanban.html");
	});

	it("Method getFrameworkTemplateURL should return the url of the template if it's a default templateUrl", function() {
		var window = {};
		window.templateUrls = [{
			package: "default",
			baseUrl: "./something/"
		}];
		window.metadataUrl = "./";
		window.serviceUrl = "../backend/service/index.php";
		var templateManager = new TemplateManager(window, zhPromise);
		var template = templateManager.getFrameworkTemplateURL("widget/kanban.html");
		expect(template).toEqual("./something/widget/kanban.html");
	});



	it("Method menuVisible should set the visibility of the menu", function() {
		templateManager.menuVisible(true);
		expect(templateManager.project.menuVisible).toBe(true);

		templateManager.menuVisible(false);
		expect(templateManager.project.menuVisible).toBe(false);
	});

	it("The constructor should create a mapper for all json packages and the getMappedPackages should return the mapped object", function() {
		var packages = templateManager.getMappedPackages();
		expect(packages).toBeDefined();
		expect(Util.isArray(packages.default)).toBe(true);
		expect(packages.default.length).toEqual(1);
		expect(packages.default[0]).toBe("./");
		expect(packages.identity).toBe("http://identity.zeedhi.com/frontend/");
		expect(packages.pm).toBe("http://pm.zeedhi.com/frontend/");
		expect(packages.devOps).toBe("http://devops.zeedhi.com/frontend/");
	});

	it("The constructor should create a mapper for all services packages and the getMappedServicePackages should return the mapped object", function() {
		var packages = templateManager.getMappedServicePackages();
		expect(packages).toBeDefined();
		expect(Util.isArray(packages.default)).toBe(true);
		expect(packages.default.length).toEqual(1);
		expect(packages.default[0]).toBe("../backend/service/index.php");
		expect(packages.identity).toBe("http://identity.zeedhi.com/backend/service/index.php");
		expect(packages.pm).toBe("http://pm.zeedhi.com/backend/service/index.php");
		expect(packages.devOps).toBe("http://devops.zeedhi.com/backend/service/index.php");
	});

	it("The constructor should throws an Exception when there is an invalid metadataUrls", function() {
		expect(function() {
			var mockWindow = {};
			var templateManager = new TemplateManager(mockWindow, zhPromise);
		}).toThrow();

		expect(function() {
			var mockWindow = {};
			mockWindow.templateUrl = "../../frontend/templates/dyman/";
			mockWindow.metadataUrl = "./";
			mockWindow.metadataUrls = [{
					"package": "identity",
					"baseUrl": "http://identity.zeedhi.com/frontend/",
				},
				"http://devops.zeedhi.com/frontend/"
			];

			var templateManager = new TemplateManager(mockWindow, zhPromise);
		}).toThrow();

		expect(function() {
			var mockWindow = {};
			mockWindow.templateUrl = "../../frontend/templates/dyman/";
			mockWindow.metadataUrl = "./";
			mockWindow.metadataUrls = [{
				"baseUrl": "http://identity.zeedhi.com/frontend/",
			}, {
				"package": "pm",
				"baseUrl": "http://pm.zeedhi.com/frontend/",
			}];
			var templateManager = new TemplateManager(mockWindow, zhPromise);
		}).toThrow();

		expect(function() {
			var mockWindow = {};
			mockWindow.templateUrl = "../../frontend/templates/dyman/";
			mockWindow.metadataUrl = "./";
			mockWindow.metadataUrls = [{
				"package": "identity",
				"baseUrl": "http://identity.zeedhi.com/frontend/"
			}, {
				"package": "pm"
			}];
			var templateManager = new TemplateManager(mockWindow, zhPromise);
		}).toThrow();
	});

	it("The constructor should throws an Exception when there is an invalid serviceUrls - After 2016 april 21th", function() {
		expect(function() {
			var mockWindow = {};
			mockWindow.templateUrl = "../../frontend/templates/dyman/";
			mockWindow.metadataUrl = "./";
			var templateManager = new TemplateManager(mockWindow, zhPromise);
		}).not.toThrow();

		var april21thOf2016 = 1463799600000;
		if (new Date().getTime() >= april21thOf2016) { //Sounds weird? Take a look at ZhFeatures.useMappedPackagesUrlWithStringUrl
			expect(function() {

				var mockWindow = {};
				mockWindow.templateUrl = "../../frontend/templates/dyman/";
				mockWindow.metadataUrl = "./";
				mockWindow.serviceUrls = [{
						"package": "identity",
						"baseUrl": "http://identity.zeedhi.com/frontend/",
					},
					"http://devops.zeedhi.com/frontend/"
				];
				var templateManager = new TemplateManager(mockWindow, zhPromise);

			}).toThrow();
		}

		expect(function() {
			var mockWindow = {};
			mockWindow.templateUrl = "../../frontend/templates/dyman/";
			mockWindow.metadataUrl = "./";
			mockWindow.serviceUrls = [{
				"baseUrl": "http://identity.zeedhi.com/backend/",
			}, {
				"package": "pm",
				"baseUrl": "http://pm.zeedhi.com/backend/",
			}];
			var templateManager = new TemplateManager(mockWindow, zhPromise);
		}).toThrow();

		expect(function() {
			var mockWindow = {};
			mockWindow.templateUrl = "../../frontend/templates/dyman/";
			mockWindow.metadataUrl = "./";
			mockWindow.serviceUrls = [{
				"package": "identity",
				"baseUrl": "http://identity.zeedhi.com/backend/"
			}, {
				"package": "pm"
			}];
			var templateManager = new TemplateManager(mockWindow, zhPromise);
		}).toThrow();
	});

	it("Should be maintained the compatibility with projects without packages", function() {
		var mockWindow = {};
		mockWindow.templateUrl = "../../frontend/templates/dyman/";
		mockWindow.metadataUrl = "./";
		mockWindow.serviceUrl = "./backend/";
		mockWindow.metadataUrls = [
			"http://identity.zeedhi.com/frontend/",
			"http://devops.zeedhi.com/frontend/"
		];
		mockWindow.serviceUrls = [
			"http://identity.zeedhi.com/backend/",
			"http://devops.zeedhi.com/backend/"
		];

		var templateManager = new TemplateManager(mockWindow, zhPromise);
		var packages = templateManager.getMappedPackages();
		var packagesServie = templateManager.getMappedServicePackages();

		expect(Util.isArray(packages.default)).toBe(true);
		expect(packages.default.length).toEqual(3);
		expect(packages.default[0]).toEqual("./");
		expect(packages.default[1]).toEqual("http://identity.zeedhi.com/frontend/");
		expect(packages.default[2]).toEqual("http://devops.zeedhi.com/frontend/");

		expect(packagesServie.default[1]).toEqual("http://identity.zeedhi.com/backend/");
		expect(packagesServie.default[2]).toEqual("http://devops.zeedhi.com/backend/");
	});

	it("The constructor should create a mapper for all common json files and the getCommonFilesSource should return the mapped object", function() {
		var commonFilesSource = templateManager.getCommonFilesSource();
		expect(commonFilesSource).toBeDefined();

		expect(Array.isArray(commonFilesSource.acl)).toBe(true);
		expect(commonFilesSource.acl.length).toEqual(3);
		expect(commonFilesSource.acl[0]).toEqual("./");
		expect(commonFilesSource.acl[1]).toEqual("http://identity.zeedhi.com/frontend/");
		expect(commonFilesSource.acl[2]).toEqual("http://pm.zeedhi.com/frontend/");

		expect(Array.isArray(commonFilesSource.words)).toBe(true);
		expect(commonFilesSource.words.length).toEqual(3);
		expect(commonFilesSource.words[0]).toEqual("./");
		expect(commonFilesSource.words[1]).toEqual("http://identity.zeedhi.com/frontend/");
		expect(commonFilesSource.words[2]).toEqual("http://devops.zeedhi.com/frontend/");

		expect(Array.isArray(commonFilesSource.data)).toBe(true);
		expect(commonFilesSource.data.length).toEqual(2);
		expect(commonFilesSource.data[0]).toEqual("./");
		expect(commonFilesSource.data[1]).toEqual("http://devops.zeedhi.com/frontend/");
	});

	it("The method getDocsURL should return the URL of the docs when there is a templateUrl defined, and there isn't a docsUrl.", function() {
		var window = {};
		window.templateUrl = "../../frontend/templates/dyman/";
		window.metadataUrl = "./";
		window.serviceUrl = "../backend/service/index.php";
		var templateManager = new TemplateManager(window, zhPromise);

		var template = templateManager.getDocsUrl("fileName.json");
		expect(template).toEqual("../../frontend/docs/dyman/fileName.json");
	});

	it("The method getDocsURL should return the url of the docs if there is a docsUrl", function() {
		var window = {};
		window.templateUrl = "../../frontend/templates/";
		window.docsUrl = "../../frontend/docs/dyman/";
		window.metadataUrl = "./";
		window.serviceUrl = "../backend/service/index.php";
		var templateManager = new TemplateManager(window, zhPromise);

		var template = templateManager.getDocsUrl("fileName.json");
		expect(template).toEqual("../../frontend/docs/dyman/fileName.json");
	});

	it("Method getDocsUrl should return the url of the docs if there is a custom docsUrl", function() {
		var window = {};
		window.templateUrl = "../../frontend/templates/dyman/";
		window.docsUrl = "../../frontend/docs/dyman/";
		window.docsUrls = [{
			package: "something",
			baseUrl: "./something/"
		}];
		window.metadataUrl = "./";
		window.serviceUrl = "../backend/service/index.php";
		var templateManager = new TemplateManager(window, zhPromise);

		var template = templateManager.getDocsUrl("something#fileName.json");
		expect(template).toEqual("./something/fileName.json");
	});

	it('Constructor should add Zeedhi package if it does not exist', function() {
		var window = {};
		window.templateUrl = "../../frontend/templates/";
		window.docsUrl = '../../frontend/docs/dyman/';
		window.serviceUrl = '../backend/service/index.php';
		window.metadataUrls = [
			{
				package: 'something',
				baseUrl: './something/'
			}
		];
		var templateManager = new TemplateManager(window, zhPromise);

		var packages = templateManager.getMappedPackages();
		expect(Object.keys(packages).length).toBe(3);

		expect(packages.default).toEqual(['./']);
		expect(packages.something).toBe('./something/');
		expect(packages.zeedhi).toBe('./bower_components/zeedhi-frontend/assets/');
	});

	it('Constructor should not edit Zeedhi package if it exists', function() {
		var window = {};
		window.templateUrl = "../../frontend/templates/";
		window.docsUrl = '../../frontend/docs/dyman/';
		window.serviceUrl = '../backend/service/index.php';
		window.metadataUrls = [
			{
				package: 'something',
				baseUrl: './something/'
			},
			{
				package: 'zeedhi',
				baseUrl: './not_bower_components/zeedhi-frontend/assets/'
			}
		];
		var templateManager = new TemplateManager(window, zhPromise);

		var packages = templateManager.getMappedPackages();
		expect(Object.keys(packages).length).toBe(3);

		expect(packages.default).toEqual(['./']);
		expect(packages.something).toBe('./something/');
		expect(packages.zeedhi).toBe('./not_bower_components/zeedhi-frontend/assets/');
	});

	describe("Version data download", function() {
		var templateManager;
		var $httpBackend;
		var $rootScope;
		beforeEach(inject(function(_$rootScope_, _$httpBackend_, $http) {
			$httpBackend = _$httpBackend_;
			$rootScope = _$rootScope_;

			window.templateUrl = "../../frontend/templates/dyman/";
			templateManager = new TemplateManager(window, zhPromise, null, $http);
			templateManager.project.versionFile = "./bower.json";
			templateManager.project.dependenciesFolder = "bower_components";

			$rootScope.$apply();
		}));

		it("Method updateVersionData should set the correct information about the bower and its dependencies", function(done) {
			$httpBackend.when('GET', './bower.json').respond({
				version: "1.0.0",
				name: "zeedhi-frontend",
				dependencies: {
					angular: "~1.2.29"
				}
			}, {});
			$httpBackend.when('GET', 'bower_components/angular/bower.json').respond({
				name: "angular",
				version: "1.2.29"
			}, {});
			templateManager.updateVersionData().then(function() {
				var versionData = templateManager.versionData;

				expect(versionData.name).toEqual("zeedhi-frontend");
				expect(versionData.version).toEqual("1.0.0");
				expect(versionData.dependencies.length).toEqual(1);
				expect(versionData.dependencies[0].name).toEqual("angular");
				expect(versionData.dependencies[0].version).toEqual("1.2.29");

				done();
			});

			$httpBackend.flush();

			zhPromise.when().then(function() {
				$httpBackend.flush();
			});
		});
	});

});